@@ -360,6 +360,7 @@ type SmallInt = Int32
 
 instance Binary Integer where
 
+    {-# INLINE put #-}
     put n | n >= lo && n <= hi = do
         putWord8 0
         put (fromIntegral n :: SmallInt)  -- fast path
@@ -374,6 +375,7 @@ instance Binary Integer where
      where
         sign = fromIntegral (signum n) :: Word8
 
+    {-# INLINE get #-}
     get = do
         tag <- get :: Get Word8
         case tag of
@@ -666,12 +668,26 @@ instance (Binary e) => Binary (T.Tree e) where
 -- Arrays
 
 instance (Binary i, Ix i, Binary e) => Binary (Array i e) where
-    put a = put (bounds a) >> put (elems a)
-    get = liftM2 listArray get get
+    put a = do
+        put (bounds a)
+        put (rangeSize $ bounds a) -- write the length
+        mapM_ put (elems a)        -- now the elems.
+    get = do
+        bs <- get
+        n  <- get                  -- read the length
+        xs <- replicateM n get     -- now the elems.
+        return (listArray bs xs)
 
 --
 -- The IArray UArray e constraint is non portable. Requires flexible instances
 --
 instance (Binary i, Ix i, Binary e, IArray UArray e) => Binary (UArray i e) where
-    put a = put (bounds a) >> put (elems a)
-    get = liftM2 listArray get get
+    put a = do
+        put (bounds a)
+        put (rangeSize $ bounds a) -- now write the length
+        mapM_ put (elems a)
+    get = do
+        bs <- get
+        n  <- get
+        xs <- replicateM n get
+        return (listArray bs xs)
